@@ -3,6 +3,9 @@
 ## We use this form because it can be sparse and because it is easy to combine
 ## multistage and multiphase sampling.
 ##
+## For (stratified, cluster) simple random sampling,
+## Dcheck simplifies to (n/N-1)/(n-1) or -(1-p)/(n-1)
+##
 Dcheck_strat<-function(strata, prob){
     strata<-as.numeric(strata) ## for ave()
     n<-length(strata)
@@ -29,7 +32,30 @@ Dcheck_multi<-function(id,strata,probs){
    rval
  }
 
-Dcheck_subset<-function(strata, subset, prob, withreplacement){
+
+## subsetting now happens in Dcheck_multi_subset
+## (no: need to know strata before subsetting to get sampsize.
+
+Dcheck_subset<-function(strata, prob,sampsize, withreplacement){
+    strata<-as.numeric(strata) ## for ave()
+    N<-length(strata)
+    n<-NROW(strata)
+    rval<-matrix(0, n,n)
+    #sampsize<-ave(strata,strata,FUN=length)
+    strats<-unique(strata)
+    if (!withreplacement){
+      for(strat in strats){
+        these <- strata == strat
+        ithese<-which(these)
+        rval[these,these]<- -(1-prob[ithese])/(sampsize[ithese]-1)
+      }
+    }
+    diag(rval)<-(1-prob)
+    rval
+}
+
+
+oldDcheck_subset<-function(strata, subset, prob, withreplacement){
     strata<-as.numeric(strata) ## for ave()
     N<-length(strata)
     n<-sum(subset)
@@ -47,7 +73,7 @@ Dcheck_subset<-function(strata, subset, prob, withreplacement){
     rval
   }
 
-Dcheck_multi_subset<-function(id,strata,subset,probs,withreplacement){
+oldDcheck_multi_subset<-function(id,strata,subset,probs,withreplacement){
    nstage<-NCOL(id)
    n<-sum(subset)
    rval<-matrix(0,n,n)
@@ -61,7 +87,28 @@ Dcheck_multi_subset<-function(id,strata,subset,probs,withreplacement){
        rval<- twophaseDcheck(rval, this_stage)
      }
    rval
+}
+
+
+Dcheck_multi_subset<-function(id,strata,subset,probs,withreplacement){
+   nstage<-NCOL(id)
+   n<-sum(subset)
+   rval<-matrix(0,n,n)
+   if (all(probs==1) && withreplacement)
+       return(as(diag(n),"sparseMatrix"))
+   sampsize<-NULL
+   for(stage in 1:nstage){
+       uid<-rep(FALSE,NROW(id))
+       uid[subset]<-!duplicated(id[subset,stage])
+       insubset<-rowsum(as.integer(subset),id[,stage],reorder=FALSE)>0
+       idx<-match(id[subset,stage],id[subset,stage][uid[subset]])
+       sampsize<-ave(as.numeric(id[,stage]),strata[,stage],FUN=function(i) length(unique(i)))
+       this_stage<-Dcheck_subset(strata[uid,stage],probs[uid,stage],sampsize, withreplacement)[idx,idx]
+       rval<- twophaseDcheck(rval, this_stage)
+     }
+   rval
  }
+
 twophaseDcheck<-function(Dcheck1,Dcheck2){
   as(-Dcheck1*Dcheck2+Dcheck1+Dcheck2,"sparseMatrix")
 }
@@ -164,9 +211,9 @@ twophase2<-function(id,strata=NULL, probs=NULL, fpc=NULL,
       cm<-rval$phase1$full$cluster[,m]
       if (nunique(sa)!=nunique(sa[subset]))
           stop("Some phase-2 strata have zero sampling fraction")
-      rval$usu<-ave(cm[subset],sa[subset],FUN=nunique)/ave(cm,sa,FUN=nunique)[subset]
+      rval$usu<-ave(as.numeric(cm[subset]),sa[subset],FUN=nunique)/ave(as.numeric(cm),sa,FUN=nunique)[subset]
   } else {
-      rval$usu<-drop(with(rval$phase1$sample,ave(cluster[,m], strata[,m], FUN=nunique))/rval$phase1$full$fpc$sampsize[rval$subset])
+      rval$usu<-drop(with(rval$phase1$sample,ave(as.numeric(cluster[,m]), strata[,m], FUN=nunique))/rval$phase1$full$fpc$sampsize[rval$subset])
   }
 
   if (length(rval$phase1$sample$prob)==length(d2$prob))
